import requests
from enola.base.common.huemul_http_info import HuemulHttpInfo
from enola.base.common.huemul_response_provider import HuemulResponseProvider
from enola.base.common.huemul_response_error import HuemulResponseError
import json

from enola.base.connect import Connect

class HuemulConnection:
    def __init__(self, connectObject: Connect):
        self.connectObject = connectObject
        #self.httpClient: CloseableHttpClient = HttpClients.createDefault


    #
    # get HTTP call to post method
    # @param route url
    # @param data info to be sent to post method
    # @return
    #
    def authRequest(self, route, data, orgId):
        if (self.connectObject.huemulCommon.getServiceUrl() == ""):
            raise NameError('API Url null or empty')

        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemulCommon.getServiceUrl() + route

            #add header
            headers = self.getHeaderForAuth({
                "authorization" : "Basic " + data,
                "orgId": orgId,
            })

            payload = "".format("")
            httpInfo = requests.request("POST", uriFinal, data=payload, headers=headers)
            # print(response.text)
        except Exception as e:
            print(e)
            

        value = self._getResponse(httpInfo)
        return value
        

    #
    # get HTTP call to post method
    # @param route url
    # @param queryParams query params
    # @param data info to be sent to post method
    # @return
    #
    def postRequest(self, route, data, queryParams = [], headerParams = None):
        if (self.connectObject.huemulCommon.getServiceUrl() == ""):
            raise NameError('API Url null or empty')

        routeParams = ""
        for i in range(len(queryParams)):
            routeParams = routeParams + ("?" if i == 0 else "&") + str(queryParams[0].name) + "=" + str(queryParams[0].value)
        
        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemulCommon.getServiceUrl() + route + routeParams

            #add header
            headers = self.getHeader(headerParams=headerParams)

            payload = data #"".format("")
            httpInfo = requests.request("POST", uriFinal, data=payload, headers=headers)
            # print(response.text)
        except Exception as e:
            print(e)
            
        value = self._getResponse(httpInfo)
        return value

    #
    # get HTTP call to post method
    # @param route url
    # @param queryParams query params
    # @param data info to be sent to put method
    # @return
    #
    def putRequest(self, route, data, queryParams = []):
        if (self.connectObject.huemulCommon.getServiceUrl() == ""):
            raise NameError('API Url null or empty')

        routeParams = ""
        for i in range(len(queryParams)):
            routeParams = routeParams + ("?" if i == 0 else "&") + str(queryParams[0].name) + "=" + str(queryParams[0].value)
        
        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemulCommon.getServiceUrl() + route + routeParams

            #add header
            headers = self.getHeader(headerParams=None)

            payload = data #"".format("")
            httpInfo = requests.request("PUT", uriFinal, data=payload, headers=headers)
            # print(response.text)
        except Exception as e:
            print(e)
            
        value = self._getResponse(httpInfo)
        return value

    #
    # common header for each http method
    # headerParams: Dictionary
    # @return Dictionary
    #
    def getHeaderForAuth(self, headerParams):
        dataToReturn = {
            "Accept" : "application/json",
            "content-type" : "application/json",
            "huemul-client-language" : "PYTHON",
            "huemul-client-version" : "1.0",
            "huemul-client-app" : "SERVER",
            "huemul-client-info" : "",
        }

        if (headerParams != None):
            dataToReturn.update(headerParams)

        return dataToReturn


    #
    # common header for each http method
    # headerParams: Dictionary
    # @return Dictionary
    #
    def getHeader(self, headerParams):
        dataToReturn = {
            "Accept" : "application/json",
            "content-type" : "application/json",
            "huemul-client-language" : "PYTHON",
            "huemul-client-version" : "1.0",
            "huemul-client-app" : "SERVER",
            "huemul-client-info" : "",
            "authorization": "Bearer " + self.connectObject.huemulCommon.getTokenId(),
            "orgId" : self.connectObject.huemulCommon.getOrgId()
        }

        if (headerParams != None): 
            dataToReturn.update(headerParams)

        return dataToReturn


    # transform data from api to response
    # response: HuemulHttpInfo
    # return HuemulResponseProvider
    def _getResponse(self, response):
        huemulResponse = HuemulResponseProvider()

        try:
            if (response.text != ""):
                dataFromJson = json.loads(response.text)
                huemulResponse.fromDict(**dataFromJson)
            else:
                huemulResponse.isSuccessful = False
                huemulResponse.errors.append(HuemulResponseError(errorId = "getResponseError", errorTxt = f'status {response.status_code}: response.text is empty'))
                huemulResponse.httpStatusCode = response.status_code
                huemulResponse.message = response.reason
            
            #huemulResponse.fromDict(**dataFromJson) #HuemulResponseProvider.fromJson(response.text)
        except Exception as e:
            #huemulResponse.isSuccessful = false;
            huemulResponse.errors.append(HuemulResponseError(errorId = "getResponseError", errorTxt = str(e)))
        
        return huemulResponse


#
# @author Sebastián Rodríguez Robotham
# base class to post/get http methods
# isSuccessful, 
# httpStatusCode, 
# message, 
# startDate, 
# elapsedTimeMS, 
# transactionId, 
# apiVersion, 
# errors, 
# data, 
# extraInfo, 
# endDate = ""
class HuemulResponseProvider:
    def __init__(self, **args):
        if (len(args) == 0):
            self.isSuccessful = False
            # status code: 200 OK, 500 error, etc
            self.httpStatusCode = ""
            # text to client
            self.message = "not connected"
            self.startDate = ""
            self.endDate = ""
            self.elapsedTimeMS = -1
            self.transactionId = ""
            # api response version
            self.apiVersion = ""

            #error detail
            self.errors = []
            #data detail
            self.dataRaw = ""
            #extra info detail
            self.extraInfoRaw = ""
        else:
            self.fromDict(
                isSuccessful = args["isSuccessful"] if "isSuccessful" in args else "" ,
                httpStatusCode = args["httpStatusCode"] if "httpStatusCode" in args else "" ,
                message = args["message"] if "message" in args else "" ,
                startDate = args["startDate"] if "startDate" in args else "" ,
                elapsedTimeMS = args["elapsedTimeMS"] if "elapsedTimeMS" in args else -1 ,
                transactionId = args["transactionId"] if "transactionId" in args else "" ,
                apiVersion = args["apiVersion"] if "apiVersion" in args else "" ,
                errors = args["errors"] if "errors" in args else [] ,
                data = args["data"] if "data" in args else "" ,
                extraInfo = args["extraInfo"] if "extraInfo" in args else "" ,
                endDate = args["endDate"] if "endDate" in args else "" 
            )
       
    """
    def load_huemul_response_from_json(self, json_data):
        # Define una función que filtra los campos que existen en la estructura de datos
        filtered_json_data = {k: v for k, v in json_data.items() if hasattr(HuemulResponseProvider, k)}
        self.fromDict(**json_data)
    """   


    def fromDict(self, isSuccessful, httpStatusCode, message, startDate, elapsedTimeMS, transactionId, apiVersion, errors, data, extraInfo, endDate = "", *args, **kwargs):
        self.isSuccessful = True if isSuccessful == True else False
        # status code: 200 OK, 500 error, etc
        self.httpStatusCode = httpStatusCode
        # text to client
        self.message = message
        self.startDate = startDate
        self.endDate = endDate
        self.elapsedTimeMS = elapsedTimeMS if elapsedTimeMS >= 0 else -1
        self.transactionId = transactionId
        # api response version
        self.apiVersion = apiVersion

        #error detail
        self.errors = errors
        #data detail
        self.dataRaw = data
        #extra info detail
        self.extraInfoRaw = extraInfo


from enola.base.common.auth.auth_model import AuthModel
from enola.base.common.huemul_common import HuemulCommon
from enola.base.common.huemul_error import HuemulError
from enola.base.common.huemul_logging import HuemulLogging

# authData: AuthModel
class Connect:
    def __init__(self, authData: AuthModel):
        self.authData = authData
        self.huemulLogging = HuemulLogging()
        self.huemulLogging.logMessageInfo(message = "WELCOME to Enola...")

        self._canExecute = False
        self._isOpen = True
        self._errorMessage = ""
        self.othersParams = []
        self.controlClassName = "" #: String = Invoker(1).getClassName.replace("$", "")
        self.controlMethodName = "" #: String = Invoker(1).getMethodName.replace("$", "")
        #val controlFileName: String = Invoker(1).getFileName.replace("$", "")

        #create error and common object
        self.controlError = HuemulError(orgId=authData.orgId, huemulLogging=self.huemulLogging)
        self.huemulCommon = HuemulCommon()
        
        #valida que jwtToken no sea nulo
        if (authData.jwtToken is None):
            self.huemulLogging.logMessageError(message = "jwtToken is null")
            self._errorMessage = "jwtToken is null"
            self._isOpen = False
            return

        #store credentials and token
        self.huemulCommon.setOrgId(authData.orgId)
        self.huemulCommon.setConsumerId(authData.consumerId)
        self.huemulCommon.setConsumerSecret(authData.consumerSecret)
        self.huemulCommon.setApplicationName(authData.applicationName)
        self.huemulCommon.setJwtToken(authData.jwtToken)
        self.huemulCommon.setTokenId(self.authData.jwtToken)
        self.huemulCommon.setServiceUrl(self.authData.urlService)
        

        self.huemulLogging.logMessageInfo(message = "authorized...")
        self.canExecute = True
        self.huemulLogging.logMessageInfo(message = "STARTED!!!")

        ### END START
        
    def isOpen(self):
        return self._isOpen

    #/************************************************************************************/
    #/******************  R E S U L T S    ***********************************************/
    #/************************************************************************************

    
    

    #/************************************************************************************/
    #/******************  U T I L   F U N C T I O N S    *********************************/
    #/************************************************************************************/

    #
    # true for execute, false can't execute
    # @return
    #
    def canExecute(self):
        return self._canExecute

    #
    # return error message
    # @return
    #
    def getErrorMessage(self):
        return self._errorMessage
    
    


#
# true for execute, false can't execute
# @return
#
import jwt
from enola.base.common.huemul_functions import HuemulFunctions
from enola.step import Step
from enola.base.common.auth.auth_model import AuthModel
from enola.base.enola_types import KindType, DataListModel, DataType, ErrOrWarnKind, Info, StepType
from enola.base.connect import Connect
from enola.base.enola_types import AgentModel
from enola.base.internal.enola_agent import create as create_enola_agent


#def add_one(number):
#    return number + 1

class Agent:
    def __init__(self, token, name, app_id=None, user_id=None, session_id=None, channel_id=None, ip=None, code_api=None, isTest=False, message_input: str = "", enola_id_prev:str = "", app_name:str="", user_name:str="", session_name:str="", channel_name:str=""):
        """
        Start agent Execution

        token: jwt token, this is used to identify the agent, request from Admin App
        name: name of this execution
        message_input: message received from user or to explain the execution
        app_id: id of app, this is used to identify the app who is calling
        user_id: id of external user, this is used to identify the user who is calling
        session_id: id of session of user or application, this is used to identify the session who is calling
        channel_id: web, chat, whatsapp, etc, this is used to identify the channel who is calling
        ip: ip of user or application, this is used to identify the ip who is calling
        code_api: code of api, this is used to identify the api who is calling
        isTest: true if this call is for testing purposes
        enola_id_prev: id of previous call, this is used to link agents sequence
        """
        self.name = name
        self.enola_id_prev = enola_id_prev
        self.enola_id = "" #se obtiene al finalizar la ejecución
        self.agent_deploy_id = ""
        self.message_input = message_input
        self.message_output = ""
        self.num_iteratons = 0
        self.hf = HuemulFunctions()
        #Connection data

        #decodificar jwt
        try:
            if token == "":
                raise Exception("token is empty.")
            
            decoded = jwt.decode(token, algorithms=['none'], options={'verify_signature': False})
            self.agentDeployId = decoded["sub"]
            self.orgId = decoded["orgId"]
            self.serviceAccountId = decoded["id"]
            self.serviceAccountName = decoded["displayName"]
            self.serviceAccountUrl = decoded["url"]
            
            #verify if serviceAccountUrl is empty, return error
            if not self.serviceAccountUrl:
                raise Exception("serviceAccountUrl is empty.")
            if not self.agentDeployId:
                raise Exception("agentDeployId is empty.")
            if not self.orgId:
                raise Exception("orgId is empty.")

        except jwt.ExpiredSignatureError:
            print("token expired.")
        except jwt.DecodeError:
            print("Error decoding token.")
        except jwt.InvalidTokenError:
            print("Invalid Token.")

        self.connection = Connect(AuthModel(jwtToken=token, urlService=self.serviceAccountUrl, orgId=self.orgId))
        

        #user information
        self.app_id = app_id
        self.app_name = app_name
        self.user_id = user_id
        self.user_name = user_name
        self.session_id = session_id
        self.session_name = session_name
        self.channel_id = channel_id
        self.channel_name = channel_name
        self.ip = ip
        self.code_api = code_api

        #current date
        self.date_start = self.hf.getDateForApi()
        
        #if is empty or not exist assign false
        self.isTest = isTest
        
        #save steps and informations
        self.step_list = []
        self.steps = 0
        self.first_step = self.new_step(self.name, message_input= self.message_input)

    ########################################################################################
    ###############    A G E N T   M E T H O D S     #######################################
    ########################################################################################


    def add_data_received(self, name:str, data, type:DataType):
        """
        add data received from user
        """
        self.first_step.agent_data_list.append(DataListModel(value=data, name=name, data_type=type, kind=KindType.RECEIVER)) 

    def add_data_send(self, name:str, data, type:DataType):
        """
        add data to send to user
        """
        self.first_step.agent_data_list.append(DataListModel(value=data, name=name, data_type=type, kind=KindType.SENDER)) 
        
    def add_custom_info(self, key, value):
        """
        add custom information to agent
        """
        self.first_step.info_list.append(Info(key, value))

    def add_file_link(self, name: str, url: str, type: str, sizeKb: int):
        """
        add file link to agent
        """
        self.first_step.add_file_link(name=name, url=url, type=type, sizeKb=sizeKb)

    def add_tag(self, key: str, value):
        """
        add tag to agent, this tag is used to search in Enola App
        """
        self.first_step.add_tag(key=key, value=value)

    def add_extra_info(self, key: str, value):
        """
        add extra information to agent, this can be used to test or debug
        """
        self.first_step.add_extra_info(key=key, value=value)

    def add_error(self, id: str, message: str, kind: ErrOrWarnKind):
        """
        register error to agent
        """
        self.first_step.add_error(id=id, message=message, kind=kind)

    def add_warning(self, id: str, message: str, kind: ErrOrWarnKind):
        """
        register warning to agent
        """
        self.first_step.add_warning(id=id, message=message, kind=kind)


    def finish_agent(self, successfull: bool, message_output: str ="", num_iteratons: int = 0):
        """
        register in Enola server

        """
        self.date_end = self.hf.getDateForApi()
        self.duration_in_ms = self.hf.getDifMs(self.date_start, self.date_end)
        self.first_step.num_iterations  = num_iteratons
        self.close_step_others(step=self.first_step, successfull=successfull, others_cost=0, step_id="AGENT", message_output=message_output)

        #register in server
        print(f'{self.name}: sending to server... ')
        agentModel = AgentModel(
            app_id=self.app_id, 
            app_name=self.app_name,
            enola_id_prev = self.enola_id_prev,
            user_id=self.user_id, 
            user_name=self.user_name, 
            session_id=self.session_id, 
            session_name=self.session_name,
            channel_id=self.channel_id,
            channel_name=self.channel_name,
            ip=self.ip, 
            code_api=self.code_api, 
            isTest=self.isTest, 
            step_list=self.step_list, 
            steps=self.steps
        )

        #print("paso 10")
        enola_result = create_enola_agent(agentModel=agentModel, connection=self.connection, raiseErrorIfFail=True)
        #show results
        if (enola_result.successfull):
            #obtiene url para evaluación
            #obtiene id de enola
            self.enola_id = enola_result.enolaId
            self.agent_deploy_id = enola_result.agentDeployId


            print(f'{self.name}: finish OK! ')
        else:
            print(f'{self.name}: finish with error: {enola_result.message}')

        return enola_result


    ########################################################################################
    ###############    S T E P   I N F O     ###############################################
    ########################################################################################


    def new_step(self, name: str, message_input: str = ""):
        """
        start new step
        name: name of this step
        message_input: message received from user or to explain the execution
        """
        #current_step = 
        self.steps += 1
        return Step(name=name, message_input=message_input)

    def close_step_token(self, step: Step, successfull: bool, message_output: str ="", token_input_num: int=0, token_output_num: int=0, token_total_num: int=0, token_input_cost: float=0, token_output_cost: float=0, token_total_cost: float=0, enola_id: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with token information
        enola_id: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id = enola_id
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.TOKEN
        step.token.token_input = token_input_num
        step.token.token_output = token_output_num
        step.token.token_total = token_total_num
        step.cost.token_input = token_input_cost
        step.cost.token_output = token_output_cost
        step.cost.token_total = token_total_cost

        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_video(self, step: Step, successfull: bool, message_output: str ="", video_num: int=0, video_sec: int=0, video_size: int=0, video_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with video information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.VIDEO
        step.video.num_videos = video_num
        step.video.sec_videos = video_sec
        step.video.size_videos = video_size
        step.cost.videos = video_cost
        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)
        
    def close_step_audio(self, step: Step, successfull: bool, message_output: str ="", audio_num:int = 0, audio_sec:int = 0, audio_size:int = 0, audio_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with audio information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.AUDIO
        step.audio.num_audio = audio_num
        step.audio.sec_audio = audio_sec
        step.audio.size_audio = audio_size
        step.cost.audio = audio_cost
        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_image(self, step: Step, successfull: bool, message_output: str ="", image_num:int = 0, image_size:int = 0, image_cost: float = 0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with image information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.IMAGE
        step.image.num_images = image_num
        step.image.size_images = image_size
        step.cost.images = image_cost
        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_doc(self, step: Step, successfull: bool, message_output: str ="", doc_num:int=0, doc_pages:int=0, doc_size:int = 0, doc_char:int=0, doc_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with doc information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.DOCUMENT
        step.doc.num_docs = doc_num
        step.doc.num_pages = doc_pages
        step.doc.size_docs = doc_size
        step.doc.num_char = doc_char
        step.cost.docs = doc_cost
        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_others(self, step: Step, successfull: bool, message_output: str ="", others_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with others information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.stepType = StepType.OTHER
        step.cost.others = others_cost
        step.date_end = self.hf.getDateForApi()
        step.duration_in_ms = self.hf.getDifMs(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    

    def __str__(self):
        return f'Agente: {self.name}, Pasos: {self.steps}'




from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_response_to_bloc import HuemulResponseToBloc
from enola.base.connect import Connect
from enola.base.enola_types import AgentModel
from enola.base.internal.enola_agent_provider import EnolaAgentProvider

class EnolaAgentBloc():
    #
    # start enolaAgentCreate
    # @param AgentModel AgentModel
    # @return HuemulResponseBloc[EnolaAgentResponseModel]
    #
    def enolaAgentCreate(self, agentModel: AgentModel, connectObject: Connect):
        continueInLoop = True
        attempt = 0
        #result = HuemulResponseToBloc(connectObject=connectObject)

        while (continueInLoop):
            result = EnolaAgentProvider(connectObject=connectObject).enolaAgentCreate(
                    agentModel=agentModel
            )
            attempt +=1
            continueInLoop = result.analyzeErrors(attempt)
        
        return result


from enola.base.connect import Connect
from enola.base.enola_types import AgentResponseModel
from enola.base.enola_types import AgentModel
from enola.base.internal.enola_agent_bloc import EnolaAgentBloc


def create(agentModel: AgentModel, connection: Connect, raiseErrorIfFail = True):
    #print("paso 20")
    if (not connection.canExecute):
        connection.huemulLogging.logMessageError(message = "cant execute: ")
        return AgentResponseModel(
            enolaId = "",
            agentDeployId = "",
            enolaApiFeedback="",
            enolaUrlFeedback="",
            successfull = False,
            message = "cant execute:"
        )

    #print("paso 30")
    connection.huemulLogging.logMessageInfo(message = "creating Enola Agent")

    enolaAgentResult = EnolaAgentBloc().enolaAgentCreate(agentModel=agentModel,connectObject=connection)
    #print("paso 40")
    #if error
    if (not enolaAgentResult.isSuccessful):
        #print("paso 50")
        print(enolaAgentResult)
        connection._canExecute = False
        try:
            connection._errorMessage = enolaAgentResult.message if (len(enolaAgentResult.errors) == 0) else enolaAgentResult.errors[0]["errorTxt"]
        except:
            connection._errorMessage = enolaAgentResult.message if (len(enolaAgentResult.errors) == 0) else enolaAgentResult.errors[0].errorTxt

        #print("paso 60")

        connection.huemulLogging.logMessageError(message = "error in enolaAgent: " + connection._errorMessage)

        if (raiseErrorIfFail):
            connection.huemulLogging.logMessageError(message = "error" + connection._errorMessage)
            raise NameError(connection._errorMessage)
        else:
            connection.huemulLogging.logMessageError(message = "error" + connection._errorMessage)
            return AgentResponseModel(
                enolaId = "",
                agentDeployId = "",
                enolaApiFeedback="",
                enolaUrlFeedback="",
                successfull = False,
                message = "error" + connection._errorMessage
            )

    #if all ok, continue
    # connectObject._processExecStepId = enolaAgentResult.data[0].processExecStepId

    return AgentResponseModel(
        #enolaId = enolaAgentResult.data["enolaId"],
        #agentDeployId = enolaAgentResult.data["agentDeployId"],
        #enolaApiFeedback = enolaAgentResult.data.agentApiFeedback if hasattr(enolaAgentResult.data, "agentApiFeedback") else "",
        #enolaUrlFeedback = enolaAgentResult.data.enolaUrlFeedback if hasattr(enolaAgentResult.data, "agentUrlFeedback") else "",
        successfull = enolaAgentResult.isSuccessful,
        message = enolaAgentResult.message,
        **enolaAgentResult.data
    )


#
# @author Sebastián Rodríguez Robotham
# receive some params
# @param name param name
# @param value param value
#
class HuemulParam:
    def __init__(self, name, value):
        self.name = name
        self.value = value


import json
from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_response_error import HuemulResponseError
from enola.base.common.huemul_response_to_bloc import HuemulResponseToBloc
from enola.base.enola_types import AgentModel, AgentResponseModel

class EnolaAgentProvider(HuemulResponseToBloc):
    
    #
    # EnolaAgentCreate
    # @param AgentModel AgentModel
    # @return AgentExecuteResponseModel[AgentExecuteResponseModel]
    #
    def enolaAgentCreate(self, agentModel: AgentModel):
        #self = AgentExecuteResponseModel()
        try:
            #print("paso 600")
            hf = HuemulFunctions()
            #print("paso 700")
            hf.deleteArgs(agentModel)
            #print("paso 800")
            #dataIn2 = jsonpickle.encode(agentModel)
            dataIn = json.dumps(agentModel.to_json(), default=lambda o: o.__dict__)
            #dataIn =  agentModel.to_json()

            #print(agentModel.__dict__)
            #dataIn = json.dumps(agentModel, default=lambda obj: obj.__dict__)
            self.message = "starting postRequest"
            huemulResponse = HuemulConnection(connectObject=self.connectObject).postRequest(
                route = "agent/execute/v1/",
                data = dataIn,
            )

            #print("paso 900")
            #get status from connection
            self.message = "starting fromResponseProvider"
            self.fromResponseProvider(huemulResponseProvider = huemulResponse)
            if (self.isSuccessful):
                self.data = [] if len(huemulResponse.dataRaw) == 0 else list(map(lambda x: AgentResponseModel(**x) ,huemulResponse.dataRaw))
        except Exception as e:
            self.errors.append(
                HuemulResponseError(errorId = "APP-101", errorTxt = str(e))
            )

        return self


from enum import Enum
import json

class StepType(Enum):
    TOKEN = "TOKEN"
    VIDEO = "VIDEO"
    AUDIO = "AUDIO"
    IMAGE = "IMAGE"
    DOCUMENT = "DOCUMENT"
    OTHER = "OTHER"

class DataType(Enum):
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    DATE = "DATE"
    BOOLEAN = "BOOLEAN"

class KindType(Enum):
    RECEIVER = "RECEIVER"
    SENDER = "SENDER"

class ErrorType(Enum):
    ERROR = "ERROR"
    WARNING = "WARNING"

class ErrOrWarnKind(Enum):
    """
    EXTERNAL: external agent call generate an unexpected error or warning
    """
    EXTERNAL = "EXTERNAL"
    """
    INTERNAL_CONTROLLED: internal agent call generate an unexpected error or warning
    """
    INTERNAL_CONTROLLED = "INTERNAL_CONTROLLED"
    """
    INTERNAL_TOUSER: controlled error or warning to send to user
    """
    INTERNAL_TOUSER = "INTERNAL_TOUSER"


class ErrorOrWarnModel:
    def __init__(self, id: str, message: str, error_type: ErrorType, kind: ErrOrWarnKind):
        self.id = id
        self.error = message
        self.error_type: ErrorType = error_type
        self.kind: ErrOrWarnKind = kind

    def to_json(self):
        return {
            "id": self.id,
            "error": self.error,
            "error_type": self.error_type.value,
            "kind": self.kind.value
        }
    

class DataListModel:
    def __init__(self, kind: str, name: str, data_type: DataType, value):
        self.kind = kind
        self.name = name
        self.data_type = data_type
        self.value = value

    def to_json(self):
        return {
            "kind": self.kind,
            "name": self.name,
            "data_type": self.data_type.value,
            "value": self.value
        }

class Info:

    def is_numeric(self, value):
        return isinstance(value, (int, float, complex))
    def is_string(self, value):
        return isinstance(value, (str))
    def is_dict(self, value):
        return isinstance(value, (dict))

    def __init__(self, type: str, key: str, value):
        self.type = type
        self.key = key
        #si valor es numerico, asignar
        if (self.is_numeric(value)):
            self.value = value
        elif (self.is_string(value)):
            if (self.is_dict(value)):
                self.value = json.dumps(value)
            else:
                self.value = value
        else:
            self.value = value
        

    def to_json(self):
        return {
            "type": self.type,
            "key": self.key,
            "value": self.value
        }

class ApiDataModel:
    def __init__(self, name: str, method: str, url: str, body: str, header: str, payload: str, description: str):
        self.name = name
        self.method = method
        self.url = url
        self.description = description
        self.body = body
        self.header = header
        self.payload = payload


    def to_json(self):
        return {
            "name": self.name,
            "method": self.method,
            "url": self.url,
            "description": self.description,
            "body": self.body,
            "header": self.header,
            "payload": self.payload
        }

class FileInfoModel:
    def __init__(self, name: str, url: str, type: str, sizeKb: int, description: str):
        self.name = name
        self.url = url
        self.type = type
        self.size = sizeKb
        self.description = description

    def to_json(self):
        return {
            "name": self.name,
            "url": self.url,
            "type": self.type,
            "size": self.size,
            "description": self.description
        }

class DataListModel:
    def __init__(self, kind: KindType, name: str, data_type: DataType, value):
        self.kind = kind
        self.name = name
        self.data_type = data_type
        self.value = value

    def to_json(self):
        return {
            "kind": self.kind.value,
            "name": self.name,
            "data_type": self.data_type.value,
            "value": self.value
        }
    

class AgentModel:
    """
    AgentModel
    """
    def __init__(self, app_id: str, user_id: str, session_id: str, channel_id: str, ip: str, code_api: str, isTest: bool, step_list: list, steps: int, enola_id_prev: str, app_name: str, user_name: str, session_name: str , channel_name: str):
        self.app_id = app_id
        self.app_name = app_name
        self.user_id = user_id
        self.user_name = user_name
        self.session_id = session_id
        self.session_name = session_name
        self.channel_id = channel_id
        self.channel_name = channel_name
        self.ip = ip
        self.code_api = code_api
        self.isTest = isTest
        #step_list es una lista, generar un arreglo ejecutando el metodo to_json de cada elemento de la lista con map
        self.step_list = step_list
        self.steps = steps
        self.enola_id_prev = enola_id_prev

    def to_json(self):
        return {
            "app_id": self.app_id,
            "app_name": self.app_name,
            "user_id": self.user_id,
            "user_name": self.user_name,
            "session_id": self.session_id,
            "channel_id": self.channel_id,
            "session_name": self.session_name,
            "channel_name": self.channel_name,
            "ip": self.ip,
            "code_api": self.code_api,
            "isTest": self.isTest,
            "step_list": list(map(lambda x: x.to_json(), self.step_list)),
            "steps": self.steps,
            "enola_id_prev": self.enola_id_prev
        }

class AgentResponseModel:
    def __init__(self, enolaId: str, agentDeployId:str, successfull: bool, message: str, enolaApiFeedback: str="", enolaUrlFeedback: str="", **args):
        self.enolaId = enolaId
        self.agentDeployId = agentDeployId
        self.enolaApiFeedback = enolaApiFeedback
        self.enolaUrlFeedback = enolaUrlFeedback
        self.successfull = successfull
        self.message = message
        self.args = args

    def to_json(self):
        return {
            "enolaId": self.enolaId,
            "agentDeployId": self.agentDeployId,
            "successfull": self.successfull,
            "message": self.message,
            "args": self.args
        }






# consumerId: String,
# consumerSecret: String,
# orgId: String,
# applicationName: String,
# urlService: String
class AuthModel:
    def __init__(self, consumerId:str = "", consumerSecret:str = "", orgId:str = "", applicationName:str = "", urlService:str = "", sessionId:str = "", jwtToken: str = ""):
        self.consumerId = consumerId
        self.consumerSecret = consumerSecret
        self.orgId = orgId
        self.applicationName = applicationName
        self.urlService = urlService
        self.sessionId = sessionId
        self.jwtToken = jwtToken
        


#
# @author Sebastián Rodríguez Robotham
# @param body http return body
# @param httpCode http return code
#
class HuemulHttpInfo:
    def __init__(self, body, httpCode):
        self.body = body
        self.httpCode = httpCode


from enola import agent
from enola.base.enola_types import ErrOrWarnKind
#from enola.base.enola_types import AgentResponseModel

token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJFTk9MQV9IVUVNVUwwNC1iMTE2NjRkMWI5NmQxZGYzZjdlMzk1YWUzMzc0ZDI3MyIsImlkIjoiOWNiZTk5ZmMtZmY0Yy00MGQyLTgzYzktZDkxMmYxZThiYjM4IiwiZGlzcGxheU5hbWUiOiJ4eHgiLCJ1cmwiOiJodHRwczovL2FwaXNlbmQuZW5vbGEtYWkuY29tL2FwaSIsIm9yZ0lkIjoiRU5PTEFfSFVFTVVMMDQiLCJpYXQiOjE3MDg5NzYzMDcsImV4cCI6MTgzNTA2MDM5OSwiaXNzIjoiZW5vbGEifQ.MSuOCT0VLZTjLeL3kqiz9jl1G-coq3c3tAhbjXLft7M"
myAgent = agent.Agent(token=token, name="Ejecución Uno", isTest=True, message_input="hola")


step1 = myAgent.new_step("step 1", message_input="como estás")

step1.add_extra_info("info1", 10)
step1.add_extra_info("info2", "valor2")

step1.add_error(id="10", message="error 1", kind=ErrOrWarnKind.INTERNAL_TOUSER)
step1.add_error(id="20", message="error 2", kind=ErrOrWarnKind.EXTERNAL)

step1.add_api_data(name="invoca algo", method="post", url="https://algo.com",  bodyToSend="bodyToSend1", headerToSend="headerToSend1", payloadReceived= "payloadReceived1", description="descripcion de la llamada")
step1.add_api_data(name="invoca 2", method="get", url="https://otrolink.com", bodyToSend="bodyToSend2", headerToSend="headerToSend2", payloadReceived= "payloadReceived2")

step1.add_file_link(name="file1", url="http://file1", sizeKb=10, type="txt")
step1.add_file_link(name="file2", url="http://file2", sizeKb=20, type="pdf")

step1.add_tag("tag1", 1)
step1.add_tag("tag2", "abc")

step1.add_warning(id="w1", message="warning1", kind=ErrOrWarnKind.EXTERNAL )
step1.add_warning(id="w2", message="warning2", kind=ErrOrWarnKind.INTERNAL_CONTROLLED )

step2 = myAgent.new_step("step 2")
step2.add_extra_info("info3", 10)
step2.add_extra_info("info4", "valor2")

step2.add_error(id="30", message="error 1", kind=ErrOrWarnKind.INTERNAL_TOUSER)
step2.add_error(id="40", message="error 2", kind=ErrOrWarnKind.EXTERNAL)

step2.add_api_data(name="invoca algo", method="post", url="https://algo.com",  bodyToSend="bodyToSend1", headerToSend="headerToSend1", payloadReceived= "payloadReceived1", description="descripcion de la llamada")
step2.add_api_data(name="invoca 2", method="get", url="https://otrolink.com", bodyToSend="bodyToSend2", headerToSend="headerToSend2", payloadReceived= "payloadReceived2")

step2.add_file_link(name="file3", url="http://file1", sizeKb=10, type="txt")
step2.add_file_link(name="file4", url="http://file2", sizeKb=20, type="pdf")

step2.add_tag("tag3", 1)
step2.add_tag("tag4", "abc")

step2.add_warning(id="w3", message="warning1", kind=ErrOrWarnKind.EXTERNAL )
step2.add_warning(id="w4", message="warning2", kind=ErrOrWarnKind.INTERNAL_CONTROLLED )

myAgent.close_step_audio(step=step1,successfull=True, audio_cost=50, audio_num=2, audio_sec=1000, audio_size=1400,step_id="123", message_output="dos" )

myAgent.close_step_doc(step=step2,successfull=False, step_id="456", doc_char=1000, doc_cost=2000, doc_num=3000, doc_pages=4000, doc_size=5000,  )
myAgent.app_id = "223",

resultado = myAgent.finish_agent(True, message_output="chaito")

print(resultado.agentDeployId)
print(resultado.enolaId)
print(resultado.enolaApiFeedback)
print(resultado.enolaUrlFeedback)
print(resultado.message)
print(resultado.successfull)



 # errorTrace, 
 # errorClassName, 
 # errorFileName, 
 # errorLineNumber, 
 # errorMethodName, 
 # errorMessage, 
 # errorIsError, 
 # errorCode
class HuemulError:
    def __init__(self, orgId, huemulLogging, **args):
        self.errorId = ""
        self.orgId = orgId
        self.errorDetail = ""
        self.errorWhoIs = ""
        self.errorWhoCode = ""
        self.huemulLogging = huemulLogging

        if (len(args) == 0):
            #val Invoker: Array[StackTraceElement] = new Exception().getStackTrace
            self.errorIsError = False
            self.errorCode = ""
            self.errorTrace = ""
            self.errorClassName = ""
            self.errorFileName = "" # Invoker(3).getFileName //todo: obtener dinamicamente a partir de los nombres de las clases (descartando)
            self.errorLineNumber = "" #Invoker(3).getLineNumber
            self.errorMethodName = ""
            self.errorMessage = ""
        elif (len(args) == 4):
            #val Invoker: Array[StackTraceElement] = new Exception().getStackTrace
            self.errorIsError = True
            self.errorCode = args["code"]
            self.errorTrace = ""
            self.errorClassName = args["getClassName"]
            self.errorFileName = "" # Invoker(3).getFileName //todo: obtener dinamicamente a partir de los nombres de las clases (descartando)
            self.errorLineNumber = "" #Invoker(3).getLineNumber
            self.errorMethodName = args["getMethodName"]
            self.errorMessage = args["message"]

            self.printError(self.errorMessage)
        elif len(args) > 4:
            self.errorTrace = args["errorTrace"]
            self.errorClassName = args["errorClassName"]
            self.errorFileName = args["errorFileName"]
            self.errorLineNumber = args["errorLineNumber"]
            self.errorMethodName = args["errorMethodName"]
            self.errorMessage = args["errorMessage"]
            self.errorIsError = args["errorIsError"]
            self.errorCode = args["errorCode"]


    # return boolean
    def isOK(self):
        return not (self.errorIsError)

    
    def printError(self, error):
        self.huemulLogging.logMessageError("***************************************************************")
        self.huemulLogging.logMessageError("HuemulLauncher: Error Detail")
        self.huemulLogging.logMessageError("***************************************************************")
        self.huemulLogging.logMessageError("error_ClassName: " + self.errorClassName)
        self.huemulLogging.logMessageError("error_FileName: " + self.errorFileName)
        self.huemulLogging.logMessageError("error_ErrorCode: " + self.errorCode)
        self.huemulLogging.logMessageError("error_LineNumber: " + self.errorLineNumber)
        self.huemulLogging.logMessageError("error_MethodName: " + self.errorMethodName)
        self.huemulLogging.logMessageError("error_Message: " + self.errorMessage)
        self.huemulLogging.logMessageError("error_Trace: " + self.errorTrace)

        self.huemulLogging.logMessageError("Detail")
        self.huemulLogging.logMessageError(error)




#
# @author Sebastián Rodríguez Robotham
# error class used by backend to return error info
# @param errorId error Id
# @param errorTxt error Message
#
class HuemulResponseError:
    def __init__(self, errorId, errorTxt):
        self.errorId = errorId
        self.errorTxt = errorTxt


from enola.base.common.auth.auth_service_provider import AuthServiceProvider

class AuthServiceBloc():
    #
    # start authSignInService
    # @param authModel authModel
    # @return HuemulResponseBloc[AuthServiceModel]
    #
    def authSignInService(self, authModel, connectObject):
        continueInLoop = True
        attempt = 0
        #result = HuemulResponseBloc()

        connectObject.huemulLogging.logMessageInfo("Ground Control station: " + authModel.urlService)
        connectObject.huemulCommon.setServiceUrl(value = authModel.urlService)

        while (continueInLoop):
            result = AuthServiceProvider(connectObject=connectObject).authSignInService(
                consumerId = authModel.consumerId,
                consumerSecret = authModel.consumerSecret,
                orgId = authModel.orgId,
                applicationName = authModel.applicationName
            )

            attempt +=1
            continueInLoop = result.analyzeErrors(attempt)
        

        return result